#!/bin/sh
# #
# # Copyright (c) Semarchy.  All rights reserved.
# #
# # PRODUCT Semarchy xDI Runtime embedded database migration script
# #   
# # This script migrates runtime scheduler and/or sessionLogs data from old runtimes using the embedded H2 database to the new embedded HSQL database.
# #

echo Start of the migration bash
echo Note: some command prompts will be opened during the migration process, this is normal behavior.
echo 

export MIGRATION_FOLDER="$(pwd)"
export RUNTIME_PATH
export SCHEDULER_MIGRATION_FLAG=0
export SESSIONLOGS_MIGRATION_FLAG=0
export DELETE_SCHEDULER_HSQL_DATABASE_FLAG=0
export DELETE_SESSIONLOGS_HSQL_DATABASE_FLAG=0

# # DATABASE PARAMETERS
export H2_SESSIONLOGS_LOGIN
export H2_SESSIONLOGS_PASSWORD
export HSQL_SESSIONLOGS_LOGIN
export HSQL_SESSIONLOGS_PASSWORD
export H2_SCHEDULER_LOGIN
export H2_SCHEDULER_PASSWORD
export HSQL_SCHEDULER_LOGIN
export HSQL_SCHEDULER_PASSWORD
export DELIVERY_LOG_LEVEL=0

# #  If no parameter is passed to the script, display script's parameters explanation*
if [ -z "$1" ]; then
  echo Error: the sctipt need following arguments to run:
  echo
  echo
  echo -runtimePath : the target runtime path, this argument is mandatory
  echo
  echo Only one of the following argument must be used at the same time
  echo -schedulerMigration : select the scheduler database to be migrated
  echo -sessionLogsMigration :  select the sessionLogs database to be migrated
  echo
  echo -deleteHsqlScheduler : delete the target scheduler database folder before the scheduler migration, can be only used if -schedulerMigration is passed as argument
  echo -deleteHsqlSessionLogs : delete the target sessionLogs database folder before the sessionLogs migration, can be only used if -sessionLogsMigration is passed as argument
  echo
  echo SessionLogs database connection parameters:
  echo For the following 4 parameters, do not set a parameter if its value is empty
  echo -h2SessionLogsLogin : set the login for the source H2 sessionLogs database
  echo -h2SessionLogsPassword : set the password for the source H2 sessionLogs database
  echo -hsqlSessionLogsLogin : set the login for the target HSQL sessionLogs database
  echo -hsqlSessionLogsPassword : set the password for the target HSQL sessionLogs database
  echo
  echo Scheduler database connection parameters:
  echo For the following 4 parameters, do not set a parameter if its value is empty
  echo -h2SchedulerLogin : set the login for the source H2 scheduler database
  echo -h2SchedulerPassword : set the password for the source H2 scheduler database
  echo -hsqlSchedulerLogin : set the login for the target HSQL scheduler database
  echo -hsqlSchedulerPassword : set the password for the target HSQL scheduler database 
  echo    
  echo -deliveryLogLevel : set the log level used by the migration deliveries, the default value is 0
  echo 
  exit 1
fi
# # Read script arguments
while [ $# -gt 0 ]; do
  case "$1" in 
    "-runtimePath")
      export RUNTIME_PATH="$2"
      shift
      shift
      ;;
    -schedulerMigration)
      export SCHEDULER_MIGRATION_FLAG=1
      shift
      ;;
    -sessionLogsMigration)
      export SESSIONLOGS_MIGRATION_FLAG=1
      shift
      ;;
    -deleteHsqlScheduler)
      export DELETE_SCHEDULER_HSQL_DATABASE_FLAG=1
      shift
      ;;
    -deleteHsqlSessionLogs)
      export DELETE_SESSIONLOGS_HSQL_DATABASE_FLAG=1
      shift
      ;;
    -h2SessionLogsLogin)
      export H2_SESSIONLOGS_LOGIN="$2"
      shift
      shift
      ;;
    -h2SessionLogsPassword)
      export H2_SESSIONLOGS_PASSWORD="$2"
      shift
      shift
      ;;
    -hsqlSessionLogsLogin)
      export HSQL_SESSIONLOGS_LOGIN="$2"
      shift
      shift
      ;;
    -hsqlSessionLogsPassword)
      export HSQL_SESSIONLOGS_PASSWORD="$2"
      shift
      shift
      ;;
    -h2SchedulerLogin)
      export H2_SCHEDULER_LOGIN="$2"
      shift
      shift
      ;;
    -h2SchedulerPassword)
      export H2_SCHEDULER_PASSWORD="$2"
      shift
      shift
      ;;
    -hsqlSchedulerLogin)
      export HSQL_SCHEDULER_LOGIN="$2"
      shift
      shift
      ;;
    -hsqlSchedulerPassword)
      export HSQL_SCHEDULER_PASSWORD="$2"
      shift
      shift
      ;;
    -deliveryLogLevel)
      export DELIVERY_LOG_LEVEL="$2"
      shift
      shift
      ;;
    *)
      shift
      ;;
  esac
done
# # Display and check script arguments's value
if [ "$SCHEDULER_MIGRATION_FLAG" -eq 0 ] && [ "$SESSIONLOGS_MIGRATION_FLAG" -eq 0 ]; then
  echo Error: you have to select the database to migrate using either -schedulerMigration or -sessionLogsMigration argument.
  exit 1
fi
if [ "$SCHEDULER_MIGRATION_FLAG" -eq 1 ] && [ "$SESSIONLOGS_MIGRATION_FLAG" -eq 1 ]; then
  echo Error: you have to select the database to migrate using either -schedulerMigration or -sessionLogsMigration argument.
  exit 1
fi
if [ "$SCHEDULER_MIGRATION_FLAG" -eq 1 ] && [ "$DELETE_SESSIONLOGS_HSQL_DATABASE_FLAG" -eq 1 ]; then
  echo Error: you cannot use the -schedulerMigration and -deleteHsqlSessionLogs arguments at the same time.
  exit 1
fi
if [ "$SESSIONLOGS_MIGRATION_FLAG" -eq 1 ] && [ "$DELETE_SCHEDULER_HSQL_DATABASE_FLAG" -eq 1 ]; then
  echo Error: you cannot use the -schedulerMigration and -deleteHsqlSessionLogs arguments at the same time.
  exit 1
fi
if [ -z "$RUNTIME_PATH" ]; then
  echo Error: you have to set the runtime folder path using the -runtimePath argument
  exit 1
fi
if [ ! -d "$RUNTIME_PATH" ]; then
  echo Error: the provided runtime folder path do not exist
  exit 1
fi
if [ ! -f "$RUNTIME_PATH/startengine.sh" ]; then
  echo Error: the startengine.sh script located into the provided runtime folder path do not exist
  exit 1
fi
echo 
echo The provided runtime path is: "$RUNTIME_PATH"
echo
if [ "$SCHEDULER_MIGRATION_FLAG" -eq 1 ]; then
  echo The provided scheduler migration is: enabled
else
  echo The provided scheduler migration is: disabled
fi
if [ "$SESSIONLOGS_MIGRATION_FLAG" -eq 1 ]; then
  echo The provided sessionLogs migration is: enabled
else
  echo The provided sessionLogs migration is: disabled
fi
if [ "$DELETE_SCHEDULER_HSQL_DATABASE_FLAG" -eq 1 ]; then
  echo The provided HSQL scheduler deletion is: enabled
else
  echo The provided HSQL scheduler deletion is: disabled
fi
if [ "$DELETE_SESSIONLOGS_HSQL_DATABASE_FLAG" -eq 1 ]; then
  echo The provided HSQL sessionLogs deletion is: enabled
else
  echo The provided HSQL sessionLogs deletion is: disabled
fi
echo
echo The provided H2 sessionLogs login is: "$H2_SESSIONLOGS_LOGIN"
echo The provided H2 sessionLogs password is: "$H2_SESSIONLOGS_PASSWORD"
echo The provided HSQL sessionLogs login is: "$HSQL_SESSIONLOGS_LOGIN"
echo The provided HSQL sessionLogs password is: "$HSQL_SESSIONLOGS_PASSWORD"
echo
echo The provided H2 scheduler login is: "$H2_SCHEDULER_LOGIN"
echo The provided H2 scheduler password is: "$H2_SCHEDULER_PASSWORD"
echo The provided HSQL scheduler login is: "$HSQL_SCHEDULER_LOGIN"
echo The provided HSQL scheduler password is: "$HSQL_SCHEDULER_PASSWORD"
echo
echo The provided delivery log level is: "$DELIVERY_LOG_LEVEL"
echo
export EXTERNAL_VALUE_RESOLVER_FILE_PATH="$RUNTIME_PATH/temp/internalDatabaseMigration/externalValueResolverForDatabaseConnectionInformation.txt"
# # Create the internalDatabaseMigration folder into the runtime temp folder
if [ ! -e "$RUNTIME_PATH/temp/internalDatabaseMigration" ]; then
  mkdir "$RUNTIME_PATH/temp/internalDatabaseMigration"
fi
# # Delete old simple file resolver 
if [ -f "$EXTERNAL_VALUE_RESOLVER_FILE_PATH" ]; then
  rm -rf "$EXTERNAL_VALUE_RESOLVER_FILE_PATH"
fi
# # Create a new simple file resolver with databases information
(echo h2SessionLogsLogin="$H2_SESSIONLOGS_LOGIN") >> $EXTERNAL_VALUE_RESOLVER_FILE_PATH
(echo h2SessionLogsPassword="$H2_SESSIONLOGS_PASSWORD") >> $EXTERNAL_VALUE_RESOLVER_FILE_PATH
(echo hsqlSessionLogsLogin="$HSQL_SESSIONLOGS_LOGIN") >> $EXTERNAL_VALUE_RESOLVER_FILE_PATH
(echo hsqlSessionLogsPassword="$HSQL_SESSIONLOGS_PASSWORD") >> $EXTERNAL_VALUE_RESOLVER_FILE_PATH
(echo h2SchedulerLogin="$H2_SCHEDULER_LOGIN") >> $EXTERNAL_VALUE_RESOLVER_FILE_PATH
(echo h2SchedulerPassword="$H2_SCHEDULER_PASSWORD") >> $EXTERNAL_VALUE_RESOLVER_FILE_PATH
(echo hsqlSchedulerLogin="$HSQL_SCHEDULER_LOGIN") >> $EXTERNAL_VALUE_RESOLVER_FILE_PATH
(echo hsqlSchedulerPassword="$HSQL_SCHEDULER_PASSWORD") >> $EXTERNAL_VALUE_RESOLVER_FILE_PATH
echo
echo -n "Press [ENTER] to continue"
read input
echo

echo  Pre migration steps
echo
echo  Start H2 databases
echo
bash "legacy-H2-database-scripts/start-legacy-H2-database.sh" -tcpPort 55555 &
echo
if [ "$DELETE_SCHEDULER_HSQL_DATABASE_FLAG" -eq 1 ]; then
  echo     Delete existing data from HSQL scheduler database
  rm -rf   "$RUNTIME_PATH/internalDb/scheduler"
  echo     End of Deletion
fi
if [ "$DELETE_SESSIONLOGS_HSQL_DATABASE_FLAG" -eq 1 ]; then
  echo     Delete existing data from HSQL sessionLogs database
  rm -rf   "$RUNTIME_PATH/internalDb/sessionLogs"
  echo     End of Deletion
fi
echo
cd "$RUNTIME_PATH"
echo  'Start the Runtime (in order to initialize HSQL databases folders) and wait 15 seconds'
bash  "startengine.sh" &
sleep 15
echo
echo -n "Press [ENTER] to continue"
read input
echo

echo  Stop the Runtime and wait 15 seconds
bash  "stopengine.sh" -user admin -uncryptedPassword admin-password
sleep 15
echo
echo -n "Press [ENTER] to continue"
read input
echo

echo  Start HSQL databases and wait 15 seconds
bash  "start-database.sh" &
sleep 15
echo
echo -n "Press [ENTER] to continue"
read input
echo
if [ "$SCHEDULER_MIGRATION_FLAG" -eq 1 ]; then
  echo     Start the scheduler migration delivery
  bash     startdelivery.sh -name "scheduler_migration" -logLevel $DELIVERY_LOG_LEVEL -conf "$MIGRATION_FOLDER/migration_properties/migration_engineParameters.xml"
  echo     End of the scheduler migration delivery, check below for errors
fi
if [ "$SESSIONLOGS_MIGRATION_FLAG" -eq 1 ]; then
  echo     Start the sessionLogs migration delivery
  bash     startdelivery.sh -name "sessionLogs_migration" -logLevel $DELIVERY_LOG_LEVEL -conf "$MIGRATION_FOLDER/migration_properties/migration_engineParameters.xml"
  echo     End of the sessionLogs migration delivery, check below for errors
fi
echo
echo -n "Press [ENTER] to continue"
read input
echo
echo Post migration steps
echo
echo     Stop HSQL databases
bash     "stop-database.sh" &
cd "$MIGRATION_FOLDER"
echo
echo     Stop H2 databases
bash    "legacy-H2-database-scripts/stop-legacy-H2-database.sh" -tcpPort 55555 &
echo
echo You can check migration results into the "$RUNTIME_PATH/temp/internalDatabaseMigration/results" folder
echo You can close other command prompt windows
echo
echo End of the migration bash
